<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       smartmedia.is
 * @since      1.0.0
 *
 * @package    Tvgexpress
 * @subpackage Tvgexpress/admin
 */

class Tvgexpress_Admin {

	private $plugin_name;

	private $version;

	public function __construct( $plugin_name, $version ) {

		//session_start();

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/tvgexpress-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/tvgexpress-admin.js', array( 'jquery' ), $this->version, false );
        wp_localize_script( $this->plugin_name, 'tvg_ajax_change_shipment', array(
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'tvg_ajax_change_shipment_nonce' => wp_create_nonce('tvg_ajax_change_shipment')
        ));

        wp_localize_script( $this->plugin_name, 'tvgexpress_printnode_action', array(
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'tvgexpress_printnode_action_nonce' => wp_create_nonce('tvgexpress_printnode_action')
        ));


	}

    public function print_shipping(){

        $shipmentNumber = sanitize_text_field($_POST['shipment_id']);
        $printer = sanitize_text_field($_POST['printer']);

        if(!empty($shipmentNumber) && !empty($printer)){
            $printNodeApiKey = $this->decrypt(get_option('tvg_printnode_api_key'));
            if(!empty($printNodeApiKey)) {
                $printNode = new Tvg_PrintNode($printNodeApiKey);
                $service = new Tvg_Service(get_option('tvg_api_username'), $this->decrypt(get_option('tvg_api_key')), get_option('tvg_api_demo'));
                $pdf = $service->getLabels($shipmentNumber,['labelType' => "PDF"]);
                $printNode->setPrinter($printer)->printPdfBase64($pdf->response->label);

                echo json_encode(['success'=>1, 'shipmentNumber'=>$shipmentNumber, 'printer'=>$printer]);
                die();
            }
        }
        echo json_encode(['success'=>0]);
        die();
    }


	public function order_metabox() {
    
	add_meta_box('order_shipment', 
					__('TVG Xpress', 'tvgexpress'),
					array($this,'display_metabox'),
					'shop_order', 
					'side', 
					'default');
	}
	
	public function display_metabox($post){
		$order_id = $post->ID;
		$this->tvg_metbox_order_id($order_id);
	}
    protected function tvg_metbox_order_id($order_id){
	    $order = wc_get_order($order_id);
        $shipment_id = $order->get_meta('shipmentID');
        $tvg_method_id = $order->get_meta('tvg_method_id');

        $service = new Tvg_Service(get_option('tvg_api_username'), $this->decrypt(get_option('tvg_api_key')), get_option('tvg_api_demo'));
        $options = new Tvg_Options();
        if(empty($tvg_method_id) or !$options->isValidId($tvg_method_id)){
            echo "<p class='notice notice-error'>This order does not have shipping method from TVG</p>";

            echo $options->toSelectbox();

            echo '<a data-order="'.$order_id.'" class="button button-primary update_tvg_shipment" style="margin:0 auto; width:100%; text-align:center;" href="'.$create_url = add_query_arg(array('action' => 'tvgexpress_create_action', 'order_id' => $order_id), admin_url('admin-post.php')).'">'. __("Change Shipment", "tvgexpress").'</a>';

        }
        else if(!empty($order->get_meta('shipmentID'))){
            echo '<strong>'.__('Shipment number','tvgxpress').':</strong> #'.$shipment_id.'<br/>';
            echo '<strong>'.__('Pdf','tvgxpress').':</strong> <a href="'.add_query_arg(array('action' => 'tvgexpress_create_pdf_action', 'order_id' => $order_id), admin_url('admin-post.php')).'" target=_blank>'.__('Open ticket','tvgxpress').'</a><br/>';

            $printNodeApiKey = $this->decrypt(get_option('tvg_printnode_api_key'));
            if(!empty($printNodeApiKey) && $printNodeApiKey !== false){
                $printNode = new Tvg_PrintNode($printNodeApiKey);

                $printers = $printNode->getPrinters();
                if(!empty($printers) && is_array($printers)){
                    echo '<br/>';
                    echo '<select name="printer" class="order-printer-tvg-select">';
                    foreach($printers as $printer){
                        echo '<option value="'.$printer->id.'">'.$printer->name.'</option>';
                    }
                    echo '</select>';
                    echo '<br/><br/>';
                    echo '<a class="button button-primary print-tvg-shipment" data-shipmentid="'.$shipment_id.'" href="'.add_query_arg(array('action' => 'tvgexpress_printnode_action', 'shipment_id' => $shipment_id), admin_url('admin-post.php')).'" target=_blank>Prenta miða</a>';
                    echo '<span class="printer-response"></span>';
                }
            }


            echo $service->getShipmentStatusHtml($shipment_id);

        }else{
            echo '<h3>'.__('Stofna sendingu').'</h3>';
            echo '<a class="button button-primary" style="width:100%; text-align:center;" href="'.$create_url = add_query_arg(array('action' => 'tvgexpress_create_action', 'order_id' => $order_id), admin_url('admin-post.php')).'">'. __("Stofna Sendingu", "tvgexpress").'</a>';
            echo '<br/><hr style="margin-top:20px"/>';
            echo '<h3>'.__('Breyta sendingu').'</h3>';
            echo $options->toSelectbox();

            echo '<a data-order="'.$order_id.'" class="button button-primary update_tvg_shipment" style="margin:0 auto; width:100%; text-align:center;" href="'.$create_url = add_query_arg(array('action' => 'tvgexpress_create_action', 'order_id' => $order_id), admin_url('admin-post.php')).'">'. __("Breyta sendingu", "tvgexpress").'</a>';
        }
        $error = get_post_meta($order_id,'tvg_error');
        if(!empty($error)){
            if(is_array($error)){
                print '<p style="padding:15px; background:#ff0000;color:white; font-weight:bold; word-wrap:break-word">' .$error[0].'</p>';
            }
        }
    }

	public function change_shipment(){
	    $method = sanitize_text_field($_POST['tvg_method']);
	    $box = sanitize_text_field($_POST['tvg_box']);
        $order_id = sanitize_text_field($_POST['order_id']);

        if(!empty($method) && !empty($order_id)){
            $order = wc_get_order($order_id);
            if(!empty($order)) {
                $order->update_meta_data('tvg_method_id', $method);
                if ($method == "DROPP") {
                    $order->update_meta_data('tvg_postbox_id', $box);
                }
                $order->save();
                ob_start();
                $this->tvg_metbox_order_id($order_id);
                $html = ob_get_contents();
                ob_end_clean();
                echo json_encode(['success' => 1, 'html' => $html]);
                die();
            }
        }
        echo json_encode(['success'=>0]);
        die();
    }


    public function tvg_shipping_custom_field() {

        global $woocommerce, $post;

        echo '<div class="options_group">';

        woocommerce_wp_checkbox(
            array(
                'id' => '_tvg_skip_free_shipping',
                'label' => 'Óvirkja Fría Sendingu TVG',
                'class' => '',
                'desc_tip' => false,
                'description' => '',
            )
        );

        echo '</div>';

        echo '<div class="options_group">';

        woocommerce_wp_checkbox(
            array(
                'id' => '_tvg_skip_shipping',
                'label' => 'Óvirkja TVG sendingu',
                'class' => '',
                'desc_tip' => false,
                'description' => '',
            )
        );

        echo '</div>';
    }

    function tvg_shipping_custom_field_save( $post_id ){

        // Checkbox
        $woocommerce_checkbox = isset( $_POST['_tvg_skip_free_shipping'] ) ? 'yes' : 'no';
        update_post_meta( $post_id, '_tvg_skip_free_shipping', $woocommerce_checkbox );

        $woocommerce_checkbox = isset( $_POST['_tvg_skip_shipping'] ) ? 'yes' : 'no';
        update_post_meta( $post_id, '_tvg_skip_shipping', $woocommerce_checkbox );

    }

    protected function get_default_key(): string
    {
		return 'das-ist-kein-geheimer-schluessel';
	}

	protected function get_default_salt(): string
    {
		return 'das-ist-kein-geheimes-salz';
	}
	
	public function encrypt( $value ) {
		if ( ! extension_loaded( 'openssl' ) ) {
			return $value;
		}
	
		$key  = $this->get_default_key();
		$salt = $this->get_default_salt();

		$method = 'aes-256-ctr';
		$ivlen  = openssl_cipher_iv_length( $method );
		$iv     = openssl_random_pseudo_bytes( $ivlen );
	
		$raw_value = openssl_encrypt( $value . $salt, $method, $key, 0, $iv );
		if ( ! $raw_value ) {
			return false;
		}
	
		return base64_encode( $iv . $raw_value );
	}

	public function decrypt( $raw_value ) {
		if ( ! extension_loaded( 'openssl' ) ) {
			return $raw_value;
		}
	
		$key  = $this->get_default_key();
		$salt = $this->get_default_salt();

		$raw_value = base64_decode( $raw_value, true );
	
		$method = 'aes-256-ctr';
		$ivlen  = openssl_cipher_iv_length( $method );
		$iv     = substr( $raw_value, 0, $ivlen );
	
		$raw_value = substr( $raw_value, $ivlen );
	
		$value = openssl_decrypt( $raw_value, $method, $key, 0, $iv );
		if ( ! $value || substr( $value, - strlen( $salt ) ) !== $salt ) {
			return false;
		}
	
		return substr( $value, 0, - strlen( $salt ) );
	}

	public function update_or_insert_option($key,$value){
		$key_exists = get_option($key);
		if ($key_exists !== false) {
			return update_option($key, $value);
		} else {
			return add_option($key, $value);
		}
	}

    public function order_shipping_status( $order_id ){

        $order = wc_get_order($order_id);
        $shipment_id = $order->get_meta('shipmentID');
        $tvg_method_id = $order->get_meta('tvg_method_id');

        if(!empty($shipment_id) && !empty($tvg_method_id)) {
            $service = new Tvg_Service(get_option('tvg_api_username'), $this->decrypt(get_option('tvg_api_key')), get_option('tvg_api_demo'));

            $shipment = $service->getShipment($shipment_id);


            echo '<h2>TVG - Staða Sendingar</h2>';
            echo '<strong>Sendingarnúmer:</strong> #' . $shipment_id . '<br/>';
            echo '<strong>Afhendingarstaður:</strong> '.$shipment->response->shipment->Receiver->Address->AddressName.'<br/><br/>';
            echo $service->getShipmentStatusHtml($shipment_id);
        }

    }
}
