<?php

class Tvgexpress_Admin_Settings extends Tvgexpress_Admin {

	private $plugin_name;
	private $version;
	
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
	 * Add the menu items to the admin menu
	 *
	 * @since    1.0.0
	 */
	public function add_admin_menu() {
		// Main Menu Item
	  	add_menu_page(
			'TVG Xpress',
			'TVG Xpress',
			'manage_options',
			'tvg-express',
			array($this, 'display_tvg_admin_page'),
            'dashicons-cloud'
        );

        add_submenu_page(
            'tvg-express',
            'Stillingar',
            'Stillingar',
            'manage_options',
            'tvg-express',
            array($this, 'display_tvg_admin_page')
        );

		add_submenu_page(
			'tvg-express',
			'API Keys',
			'API Keys',
			'manage_options',
			'tvg-express/api-key',
			array($this, 'display_api_page')
		);
	}

	public function display_tvg_admin_page(){
		
		require_once plugin_dir_path( dirname( __FILE__ ) ) . '../admin/partials/tvgexpress-admin-display.php';
	}

	public function display_api_page(){

		$settings = array(
			'title'=> 'API',
			'action'=>'process_api_form',
			'fields'=> array(
				array(
					'name' => 'tvg_api_username',
					'label' => __('Username'),
					'value' => get_option('tvg_api_username'),
				),
				array(
					'name' => 'tvg_api_key',
					'label' => __('Password'),
					'type' => 'password',
					'value' => $this->decrypt(get_option('tvg_api_key')),
				),
				array(
					'name' => 'tvg_api_demo',
					'label' => __('Enable Test Mode','tvgxpress'),
					'type' => 'checkbox',
					'value' => get_option('tvg_api_demo'),
				),
                array(
                    'name' => 'tvg_printnode_api_key',
                    'label' => __('PrintNode API Key'),
                    'type' => 'password',
                    'value' => $this->decrypt(get_option('tvg_printnode_api_key')),
                ),
			),	
		);

		$this->settings = $settings;
		require_once plugin_dir_path( dirname( __FILE__ ) ) . '../admin/partials/tvgexpress-admin-api-display.php';
	}


	public function setup_sections() {
		add_settings_section( 'settings_section', __('Settings','tvgxpress'), array($this, 'section_callback'), 'tvg-options' );
		add_settings_section( 'sender_section', __('Sender','tvgxpress'), array($this, 'section_callback'), 'tvg-options' );
	}


	public function section_callback( $arguments ) {
		switch( $arguments['id'] ){
			case 'settings_section':
				echo '<p>'.__('General settings','tvgxpress').'</p>';
			break;
		}
	}

	public function setup_fields() {
        $prefix = "tvg_";
		$fields = array(
			//Settings
			array(
				'uid' => $prefix.'status_update',
				'label' => __('Create shipment automaticly','tvgxpress'),
				'section' => 'settings_section',
				'type' => 'select',
				'options' =>['0'=>__('Disabled')]+ wc_get_order_statuses(),
				'supplemental' => __('Status update','tvgxpress'),
				'default' => ['0']
			),
			array(
				'uid' => $prefix.'kt',
				'label' => __('Kennitölu field','tvgxpress'),
				'section' => 'settings_section',
				'type' => 'text',
				'supplemental' => __('Ef notað er custom field fyrir kennitölu','tvgxpress'),
				'default' => ''
			),
			//Sender
			array(
				'uid' => $prefix.'sender_name',
				'label' => __('Name','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender name','tvgxpress'),
				'default' => ''
			),
			array(
				'uid' => $prefix.'sender_kt',
				'label' => 'Kennitala',
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Kennitala sendanda','tvgxpress'),
				'default' => ''
			),
			array(
				'uid' => $prefix.'sender_address',
				'label' => __('Address','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender address','tvgxpress'),
				'default' => ''
			),
			array(
				'uid' => $prefix.'sender_city',
				'label' => __('City','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender city','tvgxpress'),
				'default' => ''
			),
			array(
				'uid' => $prefix.'sender_postcode',
				'label' => __('Postcode','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender postcode','tvgxpress'),
				'default' => ''
			),
			array(
				'uid' => $prefix.'sender_country_code',
				'label' => __('Country code','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender country code','tvgxpress'),
				'placeholder' => 'IS',
				'default' => 'IS'
			),
			array(
				'uid' => $prefix.'sender_contact',
				'label' => __('Contact','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender contact','tvgxpress'),
				'placeholder' => '',
				'default' => ''
			),
			array(
				'uid' => $prefix.'sender_email',
				'label' => __('Email','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender email','tvgxpress'),
				'placeholder' => '',
				'default' => ''
			),
			array(
				'uid' => $prefix.'sender_phone',
				'label' => __('Phone','tvgxpress'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender phone','tvgxpress'),
				'placeholder' => '',
				'default' => ''
			)
		);

        $printNodeApiKey = $this->decrypt(get_option('tvg_printnode_api_key'));
        if(!empty($printNodeApiKey)) {
            include_once __DIR__.'/../../includes/class-tvgexpress-printnode.php';
            $printNode = new Tvg_PrintNode($printNodeApiKey);
            $printers = $printNode->getPrinters();
            $printerOptions = [];
            foreach ($printers as $printer) {
                if(!empty($printer->id))
                $printerOptions[$printer->id] = $printer->name;
            }

            $fields[] = array(
                'uid' => $prefix.'printer',
                'label' => __('Printer', 'tvgxpress'),
                'section' => 'sender_section',
                'type' => 'select',
                'options' => $printerOptions,
                'supplemental' => __('Printer', 'tvgxpress'),
                'placeholder' => '',
                'default' => ''
            );
        }
        $this->add_fields($fields,'tvg-options');


	}

	/*public function shipping_methods(){
		$prefix = 'tvg_';

		$fields = array(
			array(
				'uid' => $prefix.'sender_phone',
				'label' => __('Phone'),
				'section' => 'sender_section',
				'type' => 'text',
				'supplemental' => __('Sender phone'),
				'placeholder' => '',
				'default' => ''
			)
		);

		$this->add_fields($fields,'tvg-options');
	}*/

	public function add_fields($fields,$group){
		// Lets go through each field in the array and set it up
		foreach( $fields as $field ) {
			add_settings_field( $field['uid'], $field['label'], array($this, 'field_callback'), 'tvg-options', $field['section'], $field );
			register_setting( $group, $field['uid'] );
		}
	}

	/**
	 * This handles all types of fields for the settings
	 *
	 * @since    1.0.0
	 */
	public function field_callback($arguments) {

		// Set our $value to that of whats in the DB
		$value = get_option( $arguments['uid'] );

		// Only set it to default if we get no value from the DB and a default for the field has been set
		if(!$value) {
			$value = $arguments['default'];
		}
		if(!isset($arguments['placeholder'])){
		    $arguments['placeholder'] = "";
        }
		// Lets do some setup based ont he type of element we are trying to display.
		switch( $arguments['type'] ){
			case 'text':
			case 'password':
			case 'number':
				printf( '<input name="%1$s" id="%1$s" type="%2$s" placeholder="%3$s" value="%4$s" />', $arguments['uid'], $arguments['type'], $arguments['placeholder'], $value );
			break;
			case 'textarea':
				printf( '<textarea name="%1$s" id="%1$s" placeholder="%2$s" rows="5" cols="50">%3$s</textarea>', $arguments['uid'], $arguments['placeholder'], $value );
				break;
			case 'select':
			case 'multiselect':
				if( ! empty ( $arguments['options'] ) && is_array( $arguments['options'] ) ){
					$attributes = '';
					$options_markup = '';
					foreach( $arguments['options'] as $key => $label ){
					    $selected = (is_array($value))?$value[ array_search( $key, $value, true ) ]:'';

						$options_markup .= sprintf( '<option value="%s" %s>%s</option>', $key, selected( $selected, $key, false ), $label );
					}
					if( $arguments['type'] === 'multiselect' ){
						$attributes = ' multiple="multiple" ';
					}
					printf( '<select name="%1$s[]" id="%1$s" %2$s>%3$s</select>', $arguments['uid'], $attributes, $options_markup );
				}
				break;
			case 'radio':
			case 'checkbox':
				if( ! empty ( $arguments['options'] ) && is_array( $arguments['options'] ) ){
					$options_markup = '';
					$iterator = 0;
					foreach( $arguments['options'] as $key => $label ){
						$iterator++;
						$is_checked = '';
						// This case handles if there is only one checkbox and we don't have anything saved yet.
						if(isset($value[ array_search( $key, $value, true ) ])) {
							$is_checked = checked( $value[ array_search( $key, $value, true ) ], $key, false );
						} else {
							$is_checked = "";
						}
						// Lets build out the checkbox
						$options_markup .= sprintf( '<label for="%1$s_%6$s"><input id="%1$s_%6$s" name="%1$s[]" type="%2$s" value="%3$s" %4$s /> %5$s</label><br/>', $arguments['uid'], $arguments['type'], $key, $is_checked, $label, $iterator );
					}
					printf( '<fieldset>%s</fieldset>', $options_markup );
				}
				break;
			}

		// If there is helper text, lets show it.
		if( array_key_exists('helper',$arguments) && $helper = $arguments['helper']) {
			printf( '<span class="helper"> %s</span>', $helper );
		}
		// If there is supplemental text lets show it.
		if( array_key_exists('supplemental',$arguments) && $supplemental = $arguments['supplemental'] ){
			printf( '<p class="description">%s</p>', $supplemental );
		}
	}

	public function submit_api_key() {
		if (isset($_POST['tvg_api_key'])) {
			$api_key = sanitize_text_field($_POST['tvg_api_key']);

            if(!empty($api_key)){
				$api_key = $this->encrypt($api_key);
			}
			
			$this->update_or_insert_option('tvg_api_key',$api_key);
		}
        if (isset($_POST['tvg_printnode_api_key'])) {
            $print_api_key = sanitize_text_field($_POST['tvg_printnode_api_key']);

            if(!empty($api_key)){
                $print_api_key = $this->encrypt($print_api_key);
            }

            $this->update_or_insert_option('tvg_printnode_api_key',$print_api_key);
        }
		if( isset($_POST['tvg_api_username'])) {
			$this->update_or_insert_option('tvg_api_username',sanitize_text_field($_POST['tvg_api_username']));
		}
		if( isset($_POST['tvg_api_demo'])) {
            $this->update_or_insert_option('tvg_api_demo', sanitize_text_field($_POST['tvg_api_demo']));
        }

		wp_safe_redirect($_SERVER['HTTP_REFERER'].'&success=1');
	}



	public function admin_notice($message) { ?>
		<div class="notice notice-success is-dismissible">
			<p><?php echo($message); ?></p>
		</div><?php
	}

    public function tvg_delete_transient(){
        delete_transient('tvg_shipping_options');
        wp_safe_redirect($_SERVER['HTTP_REFERER'].'&success=1');
    }
}