<?php

class Tvg_PrintNode {

    protected $apiKey;
    protected $printer;
    protected $_apiUrl = "https://api.printnode.com/";
    /**
     * PrintNode constructor.
     */
    public function __construct($apiKey) {
        $this->_headers = array(
            'Content-Type'=>'application/json',
            'Accept'=>'application/json',
            'Authorization'=>'basic '.base64_encode($apiKey)
        );
        $this->apiKey = $apiKey;
    }

    public function getPrinters(){
        return $this->get('printers');
    }
    public function printPdfBase64($base64){
        $params = ['printerId'=>$this->getPrinter(),'contentType'=>'pdf_base64','content'=>$base64];
        return $this->post('printjobs',$params);
    }

    /**
     * @return mixed
     */
    public function getPrinter() {
        return $this->printer;
    }

    /**
     * @param  mixed  $printer
     * @return PrintNode
     */
    public function setPrinter($printer) {
        $this->printer = $printer;
        return $this;
    }



    protected function get($url, $params = null) {
        $data = array(
            'headers' => $this->_headers
        );
        if(!empty($params)){
            $url = $url.'?'.http_build_query($params);
        }

        try{
            $response = wp_remote_get($this->_apiUrl.$url,$data);
            $code = wp_remote_retrieve_response_code($response);
            $retResponse = json_decode(wp_remote_retrieve_body( $response ));
            $this->log(
                wc_print_r(
                    array(
                        'code' => $code,
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'response' => $retResponse
                    ), true
                )
            );
            return $retResponse;
        }catch(exception $e){
            $this->log(
                wc_print_r(
                    array(
                        'code' => $e->getCode(),
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'response' => $e->getMessage()
                    ), true
                )
            );
        }
    }

    protected function post($url,$params) {

        $data = array(
            'headers' => $this->_headers,
            'body'=>wp_json_encode($params),
            'data_format' => 'body'
        );

        try{
            $response = wp_remote_post($this->_apiUrl.$url,$data);
            $code = wp_remote_retrieve_response_code($response);
            $retResponse = json_decode(wp_remote_retrieve_body( $response ));
            $this->log(
                wc_print_r(
                    array(
                        'code' => $code,
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'rsponse' => $retResponse
                    ), true
                )
            );
            return $retResponse;

        }catch(exception $e){
            $this->log(
                wc_print_r(
                    array(
                        'code' => $e->getCode(),
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'rsponse' => $e->getMessage()
                    ), true
                )
            );
        }
    }

    protected function log($message){
        if (empty($this->log)) {
            $this->log = wc_get_logger();
        }

        $this->log->debug($message, array('source' => 'tvg-xpress'));

    }
}