<?php

Class Tvg_Options {

    public $weight;
    public $length;
    public $width;
    public $height;
    public $dimension_weight;

    public $maxLength = 40;
    public $maxWidth = 30;
    public $maxHeight = 30;

    private $dimension_kg_price = 42;

    public $postcode;
    public $countryCode;
    protected $_apiUrl = "http://tvg-methods.viral.is.s3-website.eu-west-2.amazonaws.com/";//"http://tvg-methods.viral.is/test/";

    public $options = [
        0 => [
            'id'=>'DAGD',
            'code'=>'DAGD',
            'title'=>'TVG - Dagdreifing (08:00-16:00)',
            'desc'=>'Dreifing til fyrirtækja og einstaklinga frá því kl 08 til kl 16, hentar fyrirtækjum vel',
            'weight'=>[
                [
                    'id'=>1,
                    'min'=>0,
                    'max'=>10,
                    'price'=>1100
                ],
                [
                    'id'=>2,
                    'min'=>10,
                    'max'=>30,
                    'price'=>1850
                ],
                [
                    'id'=>3,
                    'min'=>30,
                    'max'=>50,
                    'price'=>2450
                ],
            ],
            'postcode'=>'H',
            'minWeight'=>0,
            'maxWeight'=>40
        ],
        1 => [
            'id'=> "DAGDH",
            'code'=> "DAGD",
            'title'=> 'TVG - Hraðsending - höfuðborgarsvæði',
            'desc' => 'Hraðsendingar til viðskiptavina á höfuðborgarsvæðinu',
            'weight'=> [
                [
                    'id'=>4,
                    'min'=>0,
                    'max'=>10,
                    'price'=>3500
                ]
            ],
            'minWeight'=>0,
            'maxWeight'=>10,
            'price'=>3500,
            'postcode'=>'H'
        ],
        2 => [
            'id'=> "DAPX",
            'code'=> "DAP",
            'title'=>"TVG - XPRESS kvöldreifing",
            'desc' => "Xpress sendar út samdægurs ef pantað er fyrir hádegi, SMS tilkynningar sendar viðskiptavinum um áætlaðan komutíma. Þægileg leið til að binda sig ekki heima.",
            'weight' => [
                [
                    'id'=>5,
                    'min'=>0,
                    'max'=>10,
                    'price'=>1100
                ],
                [
                    'id'=>6,
                    'min'=>10,
                    'max'=>30,
                    'price'=>1850
                ],
                [
                    'id'=>7,
                    'min'=>30,
                    'max'=>50,
                    'price'=>2450
                ]
            ],
            'minWeight'=>0,
            'maxWeight'=>50,
            'postcode'=>'H'
        ],
        3 => [
            'id'=>'DAPL',
            'code'=>'DAP',
            'title'=>'TVG - Landsbyggðardreifing',
            'desc' => 'Sendingin flutt á næstu Flytjandastöð, SMS sent þegar sendingin er komin.',
            'weight'=>[
                [
                    'id'=>8,
                    'min'=>0,
                    'max'=>5,
                    'price'=>1000
                ],
                [
                    'id'=>9,
                    'min'=>5,
                    'max'=>10,
                    'price'=>1250
                ],
                [
                    'id'=>10,
                    'min'=>10,
                    'max'=>20,
                    'price'=>2750
                ],
                [
                    'id'=>11,
                    'min'=>20,
                    'max'=>50,
                    'price'=>4300
                ],
                [
                    'id'=>12,
                    'min'=>50,
                    'max'=>100,
                    'price'=>5500
                ],
                [
                    'id'=>13,
                    'min'=>100,
                    'max'=>200,
                    'price'=>8700
                ],
                [
                    'id'=>14,
                    'min'=>200,
                    'max'=>300,
                    'price'=>10500
                ],
            ],
            'minWeight'=>0,
            'maxWeight'=>300,
            'postcode'=>'L'
        ],
        4 => [
            'id' => 'DAF',
            'code' => 'DAF',
            'title' => 'TVG - Heimkeyrsla utan höfuðborgarsvæðis',
            'desc' => 'Heimkeyrsla utan höfuðborgarsvæðis',
            'weight' => [
                [
                    'id'=>15,
                    'min' => 0,
                    'max' => 10,
                    'price' => 1500
                ],
                [
                    'id'=>16,
                    'min' => 10,
                    'max' => 20,
                    'price' => 1750
                ]
            ],
            'minWeight' => 0,
            'maxWeight' => 20,
            'postcode' => 'NH'
        ],
        7201 => [
            'id' => 'DROPP',
            'code' => 'DROPP',
            'title' => 'TVG - Sækja á afhendingarstöð',
            'weight'=> [
                [
                    'id'=>17,
                    'min'=>0,
                    'max'=>10,
                    'price'=>620
                ]
            ],
            'price' => 620,
            'desc' => 'Við erum með afhendingarstaði út um allt höfuðborgarsvæðið, ódýr og þægileg leið til að nálgast sendinguna. QR kóði sendur í SMS þegar sendingin er komin á afhendingarstað.',
            'postcode'=>'PB',
            'minWeight'=>0,
            'maxWeight'=>10
        ]
    ];

    public function get() {
        $data = array(
            'headers' => array(
                'Content-Type'=>'application/json',
                'Accept'=>'application/json'
            )
        );
        $response = wp_remote_get($this->_apiUrl,$data);
        return json_decode(wp_remote_retrieve_body( $response ));
    }

    public function getById($id){

        $options = $this->getOptions();

        foreach($options as $option){
            if($option['id'] == $id){
                return $option;
            }
        }
        return false;
    }

    public function getOptions(){
        //return $this->options;

        if ( false === ( $tvg_shipping_options = get_transient( 'tvg_shipping_options' ) ) ) {
            $tvg_shipping_options = json_decode(json_encode($this->get()->methods), true);

            // Put the results in a transient. Expire after 12 hours.
            set_transient( 'tvg_shipping_options', $tvg_shipping_options, 12 * HOUR_IN_SECONDS );
        }

        return $tvg_shipping_options;
    }

    public function getPrice($option){
        $weight = $this->getWeight();

        if(isset($option['weight'])){

            if(isset($option['useDimensionalWeight']) && $option['useDimensionalWeight']) {
                $dimension_weight = $this->getDimensionWeight();
                if ($dimension_weight > $weight) {
                    $weight = $dimension_weight;
                }
            }

            if($option['maxWeight'] < $weight && !empty($option['noMaxWeight'])){
                $array = end($option['weight']);
                return floatval($array['price']);
            }

            foreach($option['weight'] as $w){
                if($w['min'] <= $weight and $w['max'] >= $weight){
                    return floatval($w['price']);
                }
            }
        }

        return '';
    }

    public function getWeightClass($option){
        $weight = $this->getWeight();

        if(isset($option['weight'])){

            if(isset($option['useDimensionalWeight']) && $option['useDimensionalWeight']) {
                $dimension_weight = $this->getDimensionWeight();
                if ($dimension_weight > $weight) {
                    $weight = $dimension_weight;
                }
            }

            if($option['maxWeight'] < $weight && !empty($option['noMaxWeight'])){
                return end($option['weight']);
            }

            foreach($option['weight'] as $w){
                if($w['min'] <= $weight and $w['max'] >= $weight){
                    return $w;
                }
            }
        }
        return false;
    }

    public function isAvailable($option){
        return $this->isValidPostcode($option) && $this->isValidWeight($option) && $this->isValidCountryCode($option) && $this->isValidDimension($option);
    }

    public function isValidDimension($option){
        if($option['id'] == 'DROPP'){
            if($this->getLength() <= $this->maxLength && $this->getWidth() <= $this->maxWidth && $this->getHeight() <= $this->maxHeight){
                return true;
            }
            return false;
        }
        return true;
    }

    public function isValidPostcode($ship){
        $postcode = $this->getPostcode();
        if (empty($postcode)) {
            return true;
        }
        if(is_array($ship['postcode'])){
            if(in_array($postcode, $ship['postcode'])){
                return true;
            }else{
                return false;
            }
        }

        if ($ship['postcode'] == 'A') {
            return true;
        }
        if ($ship['postcode'] == 'H') {
            return ($postcode < 275 and ! in_array($postcode, [116, 190]));
        }
        if ($ship['postcode'] == 'L') {
            return ($postcode > 275 or in_array($postcode, [116, 190]));
        }

        if ($ship['postcode'] == 'NH') {
            return in_array($postcode, [230, 235, 240, 250, 260, 262, 276, 300, 400, 600, 603, 800, 810, 815, 820, 825]);
        }

        if ($ship['postcode'] == 'PB') {
            $boxes = $this->getPostboxes();
            foreach($boxes as $box){
                if(substr($box['postcode'],0,1) == substr($postcode,0,1)){return true;}
            }
            return false;
        }
        return false;
    }

    public function getPostboxes(){
        if ( false === ( $tvg_postboxes = get_transient( 'tvg_postboxes' ) ) ) {
            $tvg_postboxes = json_decode(json_encode($this->get()->postboxes), true);

            // Put the results in a transient. Expire after 12 hours.
            set_transient( 'tvg_postboxes', $tvg_postboxes, 12 * HOUR_IN_SECONDS );
        }
        return $tvg_postboxes;
        //require_once plugin_dir_path(dirname(__FILE__)) . 'methods/postboxes.php';
        //$boxes = new Postboxes();
        //return $boxes->getBoxes();
    }

    public function isValidWeight($option){
        $weight = $this->getWeight();

        if(!empty($option['noMaxWeight'])) {
            return true;
        }

        if(key_exists('minWeight', $option)){
            return ($option['minWeight'] <= $weight and $option['maxWeight'] >= $weight);
        }else{
            return true;
        }
    }

    public function isValidCountryCode(){
        return $this->getCountryCode() == "IS";
    }

    public function isPostbox($id){
        return $id == "DROPP";
    }

    public function setWeight($weight){$this->weight = $weight; return $this;}
    public function getWeight(){return $this->weight;}
    public function setLength($length){$this->length = $length; return $this;}
    public function getLength(){return $this->length;}
    public function setWidth($width){$this->width = $width; return $this;}
    public function getWidth(){return $this->width;}
    public function setHeight($height){$this->height = $height; return $this;}
    public function getHeight(){return $this->height;}
    public function setPostcode($postcode){$this->postcode = $postcode; return $this;}
    public function getPostcode(){return $this->postcode;}
    public function setCountryCode($countryCode){$this->countryCode = $countryCode; return $this;}
    public function getCountryCode(){return $this->countryCode;}
    public function getDimensionKgPrice(){return $this->dimension_kg_price;}

    public function isValidId($id){
        return $this->getById($id) !== false;
    }

    public function toSelectbox(){
        $tvg_settings = get_option('woocommerce_tvg_express_settings');

        $html = "<select name='tvg_shipping_method' class='tvg_shipping_method' >";
        $html .= "<option selected disabled value='0'>Velja nýjan sendingarmáta</option>";
        foreach($this->getOptions() as $option){
            if(empty($tvg_settings[$option['id'].'_'.'enabled']) || $tvg_settings[$option['id'].'_'.'enabled'] == 'yes') {

                if($option['id'] == 'DROPP'){
                    $title = (!empty($tvg_settings[$option['id'].'_'.'title']))?$tvg_settings[$option['id'].'_'.'title']:'Afhendingarstaður';
                }else{
                    $title = (!empty($tvg_settings[$option['id'].'_'.'title']))?$tvg_settings[$option['id'].'_'.'title']:$option['title'];
                }

                $html .= sprintf('<option value="%s">%s</option>', $option['id'], $title);
            }
        }
        $html .= "</select>";

        $html .= "<select name='tvg_shipping_box' class='tvg_shipping_box'>";
        foreach($this->getPostboxes() as $option){
            if($option['active'] == true){
                $html .= sprintf('<option value="%s">%s</option>',$option['postboxId'],$option['name']);
            }
        }
        $html .= "</select>";

        return $html;
    }

    public function get_dimension( $dimension ) {

        $dimension_unit = get_option( 'woocommerce_dimension_unit' );
        switch ( $dimension_unit ) {

            case 'mm' :
                return $dimension / 10.000;

            case 'in' :
                return $dimension / 0.39370;

            case 'yd' :
                return $dimension / 0.010936;

            case 'cm' :
                return $dimension;

            case 'm' :
                return $dimension / 0.010000;

            default :
                return $dimension;
        }
    }

    public function convert_weight( $weight ) {

        $weight_unit = get_option( 'woocommerce_weight_unit' );

        switch ( $weight_unit ) {

            case 'kg' :
                return $weight;

            case 'g' :
                return $weight * 0.0010000;

            case 'lbs' :
                return $weight * 0.45359237;

            case 'oz' :
                return $weight * 0.02834952;

            default :
                return $weight;
        }
    }

    public function setDimensionWeight(){
        $length = $this->getLength()/100;
        $width = $this->getWidth()/100;
        $height = $this->getHeight()/100;

        $this->dimension_weight = ($length*$width*$height)*350;
        return $this;
    }

    public function getDimensionWeight(){
        return $this->dimension_weight;
    }
}